<?php

/*************************************************************************
 * Username Change - XenConcept (c) 2017
 * All Rights Reserved.
 **************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at Try it like it buy it :)
 *************************************************************************/

namespace XenConcept\UsernameChange\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int log_id
 * @property int user_id
 * @property string old_username
 * @property string new_username
 * @property int log_date
 * @property bool is_private
 * @property string change_state
 *
 * RELATIONS
 * @property \XF\Entity\User User
 * @property \XF\Entity\ApprovalQueue ApprovalQueue
 */
class ChangeLog extends Entity
{


    public function canView(&$error = null)
    {
        /** @var \XenConcept\UsernameChange\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        return $visitor->canUsernameChange();
    }

    protected function _postSave()
    {
        $visibilityChange = $this->isStateChanged('change_state', 'visible');
        $approvalChange   = $this->isStateChanged('change_state', 'moderated');

        if ($this->isUpdate())
        {
            if ($visibilityChange == 'enter')
            {
                $user = $this->em()->find('XF:User', $this->user_id);
                $user->set('last_username_change', \XF::$time);
                $user->set('username', $this->new_username);
                $user->save();

                $this->sendMail();
            }

            if ($approvalChange == 'leave' && $this->ApprovalQueue)
            {
                $this->ApprovalQueue->delete();
            }
        }

        if ($this->isInsert())
        {
            if ($this->change_state === 'moderated')
            {
                /** @var \XF\Repository\UserAlert $alertRepo */
                $alertRepo = $this->repository('XF:UserAlert');

                $alertRepo->alert($this->User, 0, '', 'xc_username_change', $this->log_id, 'noapprove');
            }

            if ($this->change_state === 'visible')
            {
                $user = $this->em()->find('XF:User', $this->user_id);
                $user->set('last_username_change', \XF::$time);
                $user->set('username', $this->new_username);
                $user->save();

                $this->sendMail();
            }
        }

        if ($approvalChange == 'enter')
        {
            $approvalQueue = $this->getRelationOrDefault('ApprovalQueue', false);
            $approvalQueue->content_date = $this->log_date;
            $approvalQueue->save();
        }

    }

    public function sendMail()
    {
        if (\XF::visitor()->Option->receive_change_username_email)
        {
            $params = [
                'old_username' => $this->old_username,
                'new_username' => $this->new_username,
                'user' => $this->User
            ];

            \XF::mailer()->newMail()
                ->setToUser($this->User)
                ->setTemplate('xc_username_change_username_change', $params)
                ->queue();
        }
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table      = 'xc_change_username_logs';
        $structure->shortName  = 'XenConcept\UsernameChange:Changelog';
        $structure->primaryKey = 'log_id';
        $structure->contentType = 'xc_username_change';
        $structure->columns = [
            'log_id'       => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'user_id'      => ['type' => self::UINT, 'required' => true],
            'old_username' => ['type' => self::STR, 'required' => true],
            'new_username' => ['type' => self::STR,
                'required' => 'xc_username_change_please_enter_a_new_username'
            ],
            'log_date'     => ['type' => self::UINT, 'default' => \XF::$time],
            'is_private'   => ['type' => self::UINT, 'default' => 0],
            'change_state' => ['type' => self::STR, 'default' => 'moderated',
                'allowedValues' => ['visible', 'moderated']
            ],
        ];
        $structure->getters = [];
        $structure->relations = [
            'User' => [
                'entity'     => 'XF:User',
                'type'       => self::TO_ONE,
                'conditions' => 'user_id'
            ],
            'ApprovalQueue' => [
                'entity' => 'XF:ApprovalQueue',
                'type' => self::TO_ONE,
                'conditions' => [
                    ['content_type', '=', 'xc_username_change'],
                    ['content_id', '=', '$log_id']
                ],
                'primary' => true
            ],
        ];

        return $structure;
    }

}