<?php

/*************************************************************************
 * Username Change - XenConcept (c) 2017
 * All Rights Reserved.
 **************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at Try it like it buy it :)
 *************************************************************************/

namespace XenConcept\UsernameChange\Repository;

use XF\Entity\User;
use XF\Mvc\Entity\Repository;

class ChangeLog extends Repository
{
    /**
     * @param $userId
     * @return \XF\Mvc\Entity\Finder
     */
    public function findChangeLogsByContent($userId)
    {
        return $this->finder('XenConcept\UsernameChange:ChangeLog')
            ->where('user_id',  $userId)
            ->where('change_state', 'visible')
            ->setDefaultOrder('log_date', 'DESC');
    }

    public function findLastChangeLog($userId)
    {
        $finder = $this->finder('XenConcept\UsernameChange:ChangeLog');

        $finder
            ->where('user_id', $userId)
            ->setDefaultOrder('log_date', 'desc');

        return $finder->fetchOne();
    }

    public function findChangeLogs(\XF\Entity\User $user)
    {
        $finder = $this->finder('XenConcept\UsernameChange:ChangeLog');

        $visitor = \XF::visitor();

        $finder
            ->where('user_id', $user->user_id)
            ->where('change_state', 'visible')
            ->setDefaultOrder('log_date', 'desc');

        if ($user->user_id != $visitor->user_id)
        {
            $finder->where('is_private', 0);
        }

        return $finder->fetch();
    }

    public function findChangeLogsForList()
    {
        $finder = $this->finder('XenConcept\UsernameChange:ChangeLog');

        $finder
            ->where('change_state', 'visible')
            ->setDefaultOrder('log_date', 'desc')
            ->with('User');

        return $finder;
    }

    public function clearLog()
    {
        $this->db()->emptyTable('xc_change_username_logs');
    }

    /**
     * @param \XenConcept\UsernameChange\XF\Entity\User $user
     */
    public function canChangeHis(User $user)
    {
        $daysBetweenChange = $user->getDaysBetweenChange();

        $lastChange = $this->findLastChangeLog($user->user_id);
        $changeLimit = ($daysBetweenChange == -1) ? false : $daysBetweenChange;

        if (!$lastChange || !$changeLimit)
        {
            return true;
        }

        if (\XF::$time < $lastChange->log_date + 86400 * $changeLimit)
        {
            return false;
        }

        return true;
    }

    public function clearLogs($userId)
    {
        $finder = $this->finder('XenConcept\UsernameChange:ChangeLog');

        $logs = $finder
            ->where('user_id', $userId)
            ->fetch();

        foreach ($logs AS $log)
        {
            $log->delete();
        }

        return true;
    }
}